@@ -1,7 +1,7 @@
 #!/bin/bash
-# Copyright 1999-2004 Gentoo Foundation
+# Copyright 1999-2005 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.201.2.31 2005/05/04 18:00:40 genone Exp $
+# $Id: /var/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.201.2.42 2005/08/20 17:24:30 jstubbs Exp $
 
 export SANDBOX_PREDICT="${SANDBOX_PREDICT}:/proc/self/maps:/dev/console:/usr/lib/portage/pym:/dev/random"
 export SANDBOX_WRITE="${SANDBOX_WRITE}:/dev/shm:${PORTAGE_TMPDIR}"
@@ -40,9 +40,7 @@ alias restore_IFS='if [ "${old_IFS:-unset}" != "unset" ]; then IFS="${old_IFS}";
 
 OCC="$CC"
 OCXX="$CXX"
-if [ "$USERLAND" == "GNU" ]; then
-	source /etc/profile.env &>/dev/null
-fi
+source /etc/profile.env &>/dev/null
 if [ -f "${PORTAGE_BASHRC}" ]; then
 	source "${PORTAGE_BASHRC}"
 fi
@@ -352,38 +350,51 @@ unpack() {
 	[ -z "$*" ] && die "Nothing passed to the 'unpack' command"
 
 	for x in "$@"; do
-		myfail="failure unpacking ${x}"
-		echo ">>> Unpacking ${x} to $(pwd)"
-		y="${x%.*}"
-		y="${y##*.}"
+		echo ">>> Unpacking ${x} to ${PWD}"
+		y=${x%.*}
+		y=${y##*.}
 
+		myfail="${x} does not exist"
+		if [ "${x:0:2}" = "./" ] ; then
+			srcdir=""
+		else
+			srcdir="${DISTDIR}/"
+		fi
+		[ ! -s "${srcdir}${x}" ] && die "$myfail"
+
+		myfail="failure unpacking ${x}"
 		case "${x##*.}" in
 			tar)
-				tar xf "${DISTDIR}/${x}" ${tarvars} || die "$myfail"
+				tar ${tarvars} xf "${srcdir}${x}" || die "$myfail"
 				;;
 			tgz)
-				tar xzf "${DISTDIR}/${x}" ${tarvars} || die "$myfail"
+				tar ${tarvars} xzf "${srcdir}${x}" || die "$myfail"
 				;;
 			tbz2)
-				bzip2 -dc "${DISTDIR}/${x}" | tar xf - ${tarvars} || die "$myfail"
+				bzip2 -dc "${srcdir}${x}" | tar xf - ${tarvars}
+				assert "$myfail"
 				;;
-			ZIP|zip)
-				unzip -qo "${DISTDIR}/${x}" || die "$myfail"
+			ZIP|zip|jar)
+				unzip -qo "${srcdir}${x}" || die "$myfail"
 				;;
 			gz|Z|z)
 				if [ "${y}" == "tar" ]; then
-					tar xzf "${DISTDIR}/${x}" ${tarvars} || die "$myfail"
+					tar ${tarvars} xzf "${srcdir}${x}" || die "$myfail"
 				else
-					gzip -dc "${DISTDIR}/${x}" > ${x%.*} || die "$myfail"
+					gzip -dc "${srcdir}${x}" > ${x%.*} || die "$myfail"
 				fi
 				;;
 			bz2)
 				if [ "${y}" == "tar" ]; then
-					bzip2 -dc "${DISTDIR}/${x}" | tar xf - ${tarvars} || die "$myfail"
+					bzip2 -dc "${srcdir}${x}" | tar xf - ${tarvars} 
+					assert "$myfail"
 				else
-					bzip2 -dc "${DISTDIR}/${x}" > ${x%.*} || die "$myfail"
+					bzip2 -dc "${srcdir}${x}" > ${x%.*} || die "$myfail"
 				fi
 				;;
+			RAR|rar)
+				unrar x -idq "${srcdir}/${x}" || die "$myfail"
+				;;
 			*)
 				echo "unpack ${x}: file format not recognized. Ignoring."
 				;;
@@ -408,9 +419,9 @@ econf() {
 	if [ -x "${ECONF_SOURCE}/configure" ]; then
 		if [ -e /usr/share/gnuconfig/ ]; then
 			local x
-			for x in $(find ${WORKDIR} -type f -name config.guess -o -name config.sub) ; do
+			for x in $(find "${WORKDIR}" -type f '(' -name config.guess -o -name config.sub ')') ; do
 				echo " * econf: updating ${x/${WORKDIR}\/} with /usr/share/gnuconfig/${x##*/}"
-				cp /usr/share/gnuconfig/${x##*/} ${x}
+				cp -f /usr/share/gnuconfig/${x##*/} ${x}
 			done
 		fi
 
@@ -463,8 +474,8 @@ econf() {
 			--datadir=/usr/share \
 			--sysconfdir=/etc \
 			--localstatedir=/var/lib \
-			${LOCAL_EXTRA_ECONF} \
-			"$@"
+			"$@" \
+			${LOCAL_EXTRA_ECONF}
 
 		if ! "${ECONF_SOURCE}/configure" \
 			--prefix=/usr \
@@ -474,8 +485,8 @@ econf() {
 			--datadir=/usr/share \
 			--sysconfdir=/etc \
 			--localstatedir=/var/lib \
-			${LOCAL_EXTRA_ECONF} \
-			"$@" ; then
+			"$@"  \
+			${LOCAL_EXTRA_ECONF}; then
 
 			if [ -s config.log ]; then
 				echo
@@ -864,15 +875,11 @@ dyn_compile() {
 		echo "!!! that you know what you are doing... You have 5 seconds to abort..."
 		echo
 
-		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
-		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
-		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
-		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
+		for x in 1 2 3 4 5 6 7 8; do
+			echo -ne "\a"
+			LC_ALL=C sleep 0.25
+		done
 
-		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
-		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
-		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
-		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
 		sleep 3
 	fi
 
@@ -1012,61 +1019,112 @@ dyn_install() {
 
 	declare -i UNSAFE=0
 	for i in $(find "${D}/" -type f -perm -2002); do
-		UNSAFE=$(($UNSAFE + 1))
+		((UNSAFE++))
 		echo "UNSAFE SetGID: $i"
 	done
 	for i in $(find "${D}/" -type f -perm -4002); do
-		UNSAFE=$(($UNSAFE + 1))
+		((UNSAFE++))
 		echo "UNSAFE SetUID: $i"
 	done
 	
-	if [ -x /usr/bin/readelf -a -x /usr/bin/file ]; then
-		for x in $(find "${D}/" -type f \( -perm -04000 -o -perm -02000 \) ); do
-			f=$(file "${x}")
-			if [ -z "${f/*SB executable*/}" -o -z "${f/*SB shared object*/}" ]; then
-				/usr/bin/readelf -d "${x}" | egrep '\(FLAGS(.*)NOW' > /dev/null
-				if [ $? != 0 ]; then
-					if [ ! -z "${f/*statically linked*/}" ]; then
-						#uncomment this line out after developers have had ample time to fix pkgs.
-						#UNSAFE=$(($UNSAFE + 1))
-						echo -ne '\a'
-						echo "QA Notice: ${x:${#D}:${#x}} is setXid, dynamically linked and using lazy bindings."
-						echo "This combination is generally discouraged. Try: CFLAGS='-Wl,-z,now' emerge ${PN}"
-						echo -ne '\a'
-						sleep 1
-					fi
-				fi
-			fi
-		done
+	if type -p scanelf > /dev/null ; then
+		# Make sure we disallow insecure RUNPATH/RPATH's
+		# Don't want paths that point to the tree where the package was built
+		# (older, broken libtools would do this).  Also check for null paths
+		# because the loader will search $PWD when it finds null paths.
+		f=$(scanelf -qyRF '%r %p' "${D}" | grep -E "(${BUILDDIR}|: |::|^ )")
+		if [[ -n ${f} ]] ; then
+			echo -ne '\a\n'
+			echo "QA Notice: the following files contain insecure RUNPATH's"
+			echo " Please file a bug about this at http://bugs.gentoo.org/"
+			echo " For more information on this issue, kindly review:"
+			echo " http://bugs.gentoo.org/81745"
+			echo "${f}"
+			echo -ne '\a\n'
+			die "Insecure binaries detected"
+		fi
+
+		# Check for setid binaries but are not built with BIND_NOW
+		f=$(scanelf -qyRF '%b %p' "${D}")
+		if [[ -n ${f} ]] ; then
+			echo -ne '\a\n'
+			echo "QA Notice: the following files are setXid, dyn linked, and using lazy bindings"
+			echo " This combination is generally discouraged.  Try re-emerging the package:"
+			echo " LDFLAGS='-Wl,-z,now' emerge ${PN}"
+			echo "${f}"
+			echo -ne '\a\n'
+			[[ ${FEATURES/stricter} != "${FEATURES}" ]] \
+				&& die "Aborting due to lazy bindings"
+			sleep 1
+		fi
+
+		# TEXTREL's are baaaaaaaad
+		f=$(scanelf -qyRF '%t %p' "${D}")
+		if [[ -n ${f} ]] ; then
+			echo -ne '\a\n'
+			echo "QA Notice: the following files contain runtime text relocations"
+			echo " Text relocations require a lot of extra work to be preformed by the"
+			echo " dynamic linker which will cause serious performance impact on IA-32"
+			echo " and might not function properly on other architectures hppa for example."
+			echo " If you are a programmer please take a closer look at this package and"
+			echo " consider writing a patch which addresses this problem."
+			echo "${f}"
+			echo -ne '\a\n'
+			[[ ${FEATURES/stricter} != "${FEATURES}" ]] \
+				&& die "Aborting due to textrels"
+			sleep 1
+		fi
+
+		# Check for files with executable stacks
+		f=$(scanelf -qyRF '%e %p' "${D}")
+		if [[ -n ${f} ]] ; then
+			echo -ne '\a\n'
+			echo "QA Notice: the following files contain executable stacks"
+			echo " Files with executable stacks will not work properly (or at all!)"
+			echo " on some architectures/operating systems.  A bug should be filed"
+			echo " at http://bugs.gentoo.org/ to make sure the file is fixed."
+			echo "${f}"
+			echo -ne '\a\n'
+			[[ ${FEATURES/stricter} != "${FEATURES}" ]] \
+				&& die "Aborting due to +x stack"
+			sleep 1
+		fi
+
+		# Save NEEDED information
+		scanelf -qyRF '%p %n' "${D}" | sed -e 's:^:/:' > "${BUILDDIR}"/build-info/NEEDED
 	fi
 
-	if [[ $UNSAFE > 0 ]]; then
+	if [[ ${UNSAFE} > 0 ]] ; then
 		die "There are ${UNSAFE} unsafe files. Portage will not install them."
 	fi
 	
+	# dumps perms to stdout.  if error, no perms dumped.
 	function stat_perms() {
 		local f
-		if ! type -p stat &>/dev/null; then
-			do_stat() {
-				# Generic version -- Octal result
-				python -c "import os,stat; print '%o' % os.stat('$1')[stat.ST_MODE]"
-			}
-		else
-			if [ "${USERLAND}" == "BSD" ] || [ "${USERLAND}" == "Darwin" ]; then
+		# only define do_stat if it hasn't been already
+		if ! type -p do_stat &> /dev/null; then
+			if ! type -p stat &>/dev/null; then
 				do_stat() {
-					# BSD version -- Octal result
-					$(type -p stat) -f '%p' "$1"
+					# Generic version -- Octal result
+					python -c "import os,stat; print '%o' % os.stat('$1')[stat.ST_MODE]"
 				}
 			else
-				do_stat() {
-					# Linux version -- Hex result converted to Octal
-					f=$($(type -p stat) -c '%f' "$1") || return $?
-					printf '%o' "0x$f"
-				}
+				if [ "${USERLAND}" == "BSD" ] || [ "${USERLAND}" == "Darwin" ]; then
+					do_stat() {
+						# BSD version -- Octal result
+						$(type -p stat) -f '%p' "$1"
+					}
+				else
+					do_stat() {
+						# Linux version -- Hex result converted to Octal
+						f=$($(type -p stat) -c '%f' "$1") || return $?
+						printf '%o' "0x$f"
+					}
+				fi
 			fi
 		fi
 
-		f=$(do_stat "$@") || die "Failed to perform stat operation on file: $1"
+		f=$(do_stat "$@") || return
 		f="${f:2:4}"
 		echo $f
 	}
@@ -1075,9 +1133,15 @@ dyn_install() {
 	local count=0
 	find "${D}/" -user  portage | while read file; do
 		count=$(( $count + 1 ))
-		s=$(stat_perms $file)
+		if [ ! -L "${file}" ]; then
+			s=$(stat_perms "$file")
+			if [ -z "${s}" ]; then
+				ewarn "failed stat_perm'ing $file.  User intervention during install isn't wise..."
+				continue
+			fi
+		fi
 		chown root "$file"
-		chmod "$s" "$file"
+		[[ ! -L "${file}" ]] && chmod "$s" "$file"
 	done
 	if (( $count > 0 )); then
 		ewarn "$count files were installed with user portage!"
@@ -1086,13 +1150,15 @@ dyn_install() {
 	count=0
 	find "${D}/" -group portage | while read file; do
 		count=$(( $count + 1 ))
-		s=$(stat_perms "$file")
-		if [ "$USERLAND" == "BSD" ] || [ "$USERLAND" == "Darwin" ];then
-			chgrp wheel "$file"
-		else
-			chgrp root "$file"
+		if [ ! -L "${file}" ]; then
+			s=$(stat_perms "$file")
+			if [ -z "${s}" ]; then
+				echo "failed stat_perm'ing '$file' . User intervention during install isn't wise..."
+				continue
+			fi
 		fi
-		chmod "$s" "$file"
+		chgrp 0 "${file}"
+		[[ ! -L "${file}" ]] && chmod "$s" "$file"
 	done
 	if (( $count > 0 )); then
 		ewarn "$count files were installed with group portage!"
@@ -1721,9 +1787,10 @@ export TMPDIR="${T}"
 
 #turn off glob expansion from here on in to prevent *'s and ? in the DEPEND
 #syntax from getting expanded :)  Fixes bug #1473
+#check eclass rdepends also. bug #58819
 set -f
-if [ "${RDEPEND-unset}" == "unset" ]; then
-	export RDEPEND=${DEPEND}
+if [ "${RDEPEND-unset}" == "unset" ] && [ "${E_RDEPEND-unset}" == "unset" ] ; then
+	export RDEPEND="${DEPEND} ${E_DEPEND}"
 	debug-print "RDEPEND: not set... Setting to: ${DEPEND}"
 fi
 
